// Copyright 2011 Thomas Kamps anubis1@linux-ecke.de
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#include "browsertab.h"

#include <QDebug>
#include <QMessageBox>
#include <QFileInfo>

BrowserTab::BrowserTab(QObject *parent) : QObject(parent) {
	//Create tab-button menu
	menu = new QMenu();
	acShow = new QAction(tr("Show"), this);
	acBookmark = new QAction(tr("Bookmark"), this);
	acEditUrl = new QAction(tr("Edit URL"), this);
	acEditUrl->setCheckable(true);
	acEditUrl->setChecked(true);
	acClose = new QAction(tr("Close"), this);

	menu->addAction(acShow);
	menu->addAction(acBookmark);
	menu->addAction(acEditUrl);
	menu->addSeparator();
	menu->addAction(acClose);

	//Create Tab-button
	tabButton = new QToolButton();
	tabButton->setMenu(menu);
	tabButton->setDefaultAction(acShow);
	tabButton->setText(tr("Empty page"));
	tabButton->setIcon(QIcon(":/icons/new.png"));
	tabButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);

	//Create WebView layout
	content = new QWidget();
	urlBar = new QLineEdit(content);
	view = new WebView(content);
	searchBar = new SearchBar(content);
	searchBar->hide();

	layout = new QVBoxLayout(content);

	layout->setContentsMargins(0,0,0,0);
	content->setLayout(layout);
	layout->addWidget(urlBar);
	layout->addWidget(view);
	layout->addWidget(searchBar);

	view->page()->setForwardUnsupportedContent(true);

	loading = false;

	//connect the signals with the slots
	connect(view, SIGNAL(urlChanged(QUrl)), this, SLOT(showUrl(QUrl)));
	connect(view, SIGNAL(titleChanged(QString)), this, SLOT(setTitle(QString)));
	connect(view, SIGNAL(iconChanged()), this, SLOT(setIcon()));

	connect(view->page(), SIGNAL(linkHovered(QString,QString,QString)), this, SLOT(updateHoverUrl(QString)));
	connect(view->page()->networkAccessManager(), SIGNAL(sslErrors(QNetworkReply*,QList<QSslError>)), this, SLOT(setSSLError()));
	connect(view->page(), SIGNAL(windowCloseRequested()), this, SLOT(closeTab()));
	connect(view->page(), SIGNAL(downloadRequested(QNetworkRequest)), this, SLOT(startDownload(QNetworkRequest)));
	connect(view->page(), SIGNAL(unsupportedContent(QNetworkReply*)), this, SLOT(startDownload(QNetworkReply*)));

	connect(urlBar, SIGNAL(returnPressed()), this, SLOT(loadUrl()));

	//Signals from tab-button menu
	connect(acShow, SIGNAL(triggered()), this, SLOT(showTab()));
	connect(acBookmark, SIGNAL(triggered()), this, SLOT(newBookmark()));
	connect(acEditUrl, SIGNAL(triggered()), this, SLOT(showUrlBar()));
	connect(acClose, SIGNAL(triggered()), this, SLOT(closeTab()));

	connect(view, SIGNAL(newWindow(QWebView**)), this, SIGNAL(newWindow(QWebView**)));

	connect(view, SIGNAL(loadStarted()), this, SLOT(startLoading()));
	connect(view, SIGNAL(loadFinished(bool)), this, SLOT(finishLoading()));

	connect(searchBar, SIGNAL(searchFor(QString,QWebPage::FindFlags,bool*)), this, SLOT(search(QString,QWebPage::FindFlags,bool*)));
}

BrowserTab::~BrowserTab() {
	delete acShow;
	delete acEditUrl;
	delete acBookmark;

	content->deleteLater();
	tabButton->deleteLater();
	menu->deleteLater();
	acClose->deleteLater();
}

bool BrowserTab::isEmptyPage() {
	return !loading && (view->url().toString() == "about:blank");
}

void BrowserTab::gainFocus() {
	if (urlBar->isVisibleTo(content))
		urlBar->setFocus(Qt::OtherFocusReason);
}

void BrowserTab::back() {
	view->back();
}

void BrowserTab::forward() {
	view->forward();
}

void BrowserTab::stop() {
	view->stop();
}

void BrowserTab::reload() {
	view->reload();
}

void BrowserTab::showTab() {
	emit(selected(this));
}

void BrowserTab::goTo(QString url, QIcon icon, bool hideUrlBar) {
	loading = true;
	view->load(QUrl(url));
	if (!icon.isNull()) tabButton->setIcon(icon);
	else tabButton->setIcon(QWebSettings::iconForUrl(QUrl(url)));
	if (hideUrlBar) urlBar->hide();

	acEditUrl->setChecked(urlBar->isVisible());
}

void BrowserTab::showSearchBar() {
	searchBar->show();
	searchBar->gainFocus();
}

void BrowserTab::setTitle(QString title) {
	if (title == "") title = tr("Empty page");

	//Short title to 100 shown chars
	if (title.size() > 103)
		title = title.left(50)+"..."+title.right(50);
	tabButton->setText(title);
}

void BrowserTab::setIcon() {
	tabButton->setIcon(view->icon());
}

void BrowserTab::newBookmark() {
	emit(bookmark(view->title(), view->url().toString()));
}

void BrowserTab::startDownload(const QNetworkRequest &reply) {
	emit(downloadStarted(reply.url()));
}

void BrowserTab::startDownload(QNetworkReply* reply) {
	QUrl url = reply->url();
	QString file = QFileInfo(url.path()).fileName();

	//First stop downloading, we create a new download with a seperate AccessManager.
	if (reply->isOpen())
		reply->close();
	reply->deleteLater();

	//Ask user to Download the file
	int answer = QMessageBox::question(view, tr("Start download"), tr("Download %1 ?").arg(file), QMessageBox::Yes, QMessageBox::No);
	if (answer == QMessageBox::Yes) {
		emit(downloadStarted(url));
	}
}

void BrowserTab::closeTab() {
	emit(closed(this));
}

void BrowserTab::setSSL(bool sslUsed) {
	ssl = sslUsed;
	if (!sslOk) return;

	QPalette p;
	if (ssl)
		p.setBrush(QPalette::Button, QColor(160,255,160));
	tabButton->setPalette(p);
}

void BrowserTab::setSSLError() {
	sslOk = false;
	QPalette p;
	p.setBrush(QPalette::Button, QColor(255,192,192));
	tabButton->setPalette(p);
}

void BrowserTab::showUrlBar() {
	if (urlBar->isVisible()) urlBar->hide();
	else urlBar->show();

	acEditUrl->setChecked(urlBar->isVisible());
}

void BrowserTab::search(QString text, QWebPage::FindFlags flags, bool *found) {
	(*found) = view->page()->findText(text, flags);
}

void BrowserTab::showUrl(QUrl url) {
	QString urlStr = url.toString();
	setSSL(urlStr.startsWith("https"));
	if (urlStr == "about:blank") urlStr = "";
	urlBar->setText(urlStr);
}

void BrowserTab::loadUrl() {
	QString url = urlBar->text();
	if (!url.startsWith("http://") && (url != "about:blank")) url = "http://"+url;
	setSSL(url.startsWith("https"));
	view->load(QUrl(url));
	urlBar->hide();
	acEditUrl->setChecked(false);
	if (url != "about:blank")
		tabButton->setIcon(QWebSettings::iconForUrl(QUrl(url)));
	else
		tabButton->setIcon(QIcon(":/icons/new.png"));
}

void BrowserTab::openNewTab() {
	emit(newTab(curHoverUrl));
}

void BrowserTab::updateHoverUrl(QString url) {
	if (url.isNull()) {
		return;
	}
	curHoverUrl = url;
}

